@@ -2,7 +2,8 @@ from .client import Client
 from .error import ClueaiError
 
 MODELFUN_API_URL = 'https://www.modelfun.cn/modelfun/api/'
-TEXT_TO_IMAGE_URL= "http://clueai.cn/clueai/text2image?text="
+TEXT_TO_IMAGE_URL= "https://clueai.cn/clueai/text2image?text="
+CLUEAI_API_URL = "https://clueai.cn/clueai"
 MODELFUN_VERSION = '2022-08-08'
 GENERATE_URL = 'serving_api'
 CLASSIFY_URL = 'serving_api'
