@@ -15,6 +15,7 @@ import clueai
 from clueai.embeddings import Embeddings
 from clueai.error import ClueaiError
 from clueai.generation import Generations, Generation, TokenLikelihood
+from clueai.match import Match, Matches, Score
 from clueai.tokenize import Tokens
 from clueai.classify import Classifications, Classification, Example as ClassifyExample, Confidence
 from clueai.extract import Entity, Example as ExtractExample, Extraction, Extractions
@@ -46,6 +47,7 @@ class Client:
         self.api_key = api_key
         self.api_url = clueai.MODELFUN_API_URL
         self.text_2_image_api_url = clueai.TEXT_TO_IMAGE_URL
+        self.clueai_api_url = clueai.CLUEAI_API_URL
         self.num_workers = num_workers
         self.request_dict = request_dict
         if version is None:
@@ -94,6 +96,66 @@ class Client:
                 headers=response.headers)
         return res
 
+    def upload_corpus(
+        self,
+        file_path: str,
+        field: str,
+        headers: dict = {},
+        model_name: str = None
+        ):
+        tmp_headers = {
+            'Api-Key': 'BEARER {}'.format(self.api_key),
+            'Content-Type': 'application/json',
+            'Request-Source': 'python-sdk',
+            'Model-name': model_name
+        }
+        if self.modelfun_version != '':
+            tmp_headers['clueai-Version'] = self.modelfun_version
+        data_json = {
+            "field": field
+        }
+        files = {'file': open(file_path,'rb')}
+        tmp_headers.update(headers)
+
+        print(files, data_json, f"{self.clueai_api_url}/search/upload/")
+        res = requests.post(f"{self.clueai_api_url}/search/upload/", 
+            files=files, data=data_json, headers=headers)
+        
+        #key =  res.json()["key"]
+        return res.json()
+    
+    def search(
+        self, 
+        query: str,
+        engine_key: str,
+        headers: dict = {},
+        model_name: str = None
+        ):
+        tmp_headers = {
+            'Api-Key': 'BEARER {}'.format(self.api_key),
+            'Content-Type': 'application/json',
+            'Request-Source': 'python-sdk',
+            'Model-name': model_name
+        }
+        if self.modelfun_version != '':
+            tmp_headers['clueai-Version'] = self.modelfun_version
+        tmp_headers.update(headers)
+        response = requests.get(f"{self.clueai_api_url}/search/?key={engine_key}&query={query}",
+            headers=tmp_headers)
+        response = response.json()
+        result =  response['result']
+        scoreObj = []
+        for answer, score in result['answer_score'].items():
+            scoreObj.append(Score(
+                answer, float(score)
+                ))
+
+
+        match_instance = Match(result['query'], result['best_answer'], scoreObj)
+            
+
+        return Matches([match_instance])
+
     def text2image(
         self,
         prompt: str,
@@ -132,7 +194,8 @@ class Client:
         frequency_penalty: float = 0.0,
         presence_penalty: float = 0.0,
         stop_sequences: List[str] = None,
-        return_likelihoods: str = None
+        return_likelihoods: str = None,
+        headers: dict = {}
     ) -> Generations:
         json_body = json.dumps({
             'task_type': "generate",
@@ -148,7 +211,7 @@ class Client:
             'stop_sequences': stop_sequences,
             'return_likelihoods': return_likelihoods
         }, ensure_ascii=False)
-        response = self.__request(json_body, clueai.GENERATE_URL, model_name)
+        response = self.__request(json_body, clueai.GENERATE_URL, model_name, headers=headers)
         #print(f"res: {response}")
         generations: List[Generation] = []
         for gen in response['result']:
@@ -195,7 +258,6 @@ class Client:
                 confidenceObj.append(Confidence(
                     res['confidence'][i]['label'],
                     res['confidence'][i]['confidence']))
-            Classification(res['input'], res['prediction'], confidenceObj)
             classifications.append(Classification(
                 res['input'], res['prediction'], confidenceObj))
 
@@ -264,23 +326,23 @@ class Client:
                 headers=req.getAllResponseHeaders())
         return res
 
-    def __request(self, json_body, endpoint, model_name) -> Any:
-        headers = {
+    def __request(self, json_body, endpoint, model_name, headers={}) -> Any:
+        tmp_headers = {
             'Api-Key': 'BEARER {}'.format(self.api_key),
             'Content-Type': 'application/json',
             'Request-Source': 'python-sdk',
             'Model-name': model_name
         }
         if self.modelfun_version != '':
-            headers['clueai-Version'] = self.modelfun_version
-
+            tmp_headers['clueai-Version'] = self.modelfun_version
+        tmp_headers.update(headers)
         url = urljoin(self.api_url, endpoint)
         #print(json_body)
         if use_xhr_client:
-            response = self.__pyfetch(url, headers, json_body)
+            response = self.__pyfetch(url, tmp_headers, json_body)
             return response
         else:
-            response = requests.post(url, json=json.loads(json_body), headers=headers)
+            response = requests.post(url, json=json.loads(json_body), headers=tmp_headers)
             try:
                 res = json.loads(response.text)
             except Exception:
